import datetime
from django.conf import settings
from announce import AnnounceClient

announce_client = AnnounceClient()
announce_cookie_name = 'announceToken'

class AnnounceCookieMiddleware(object):
    """
    call the Announce.js API interface and get a token
    for the currently logged-in user.
    set the token received from the API as a cookie.
    
    Put this before `AuthenticationMiddleware`.
    """
    def get_token(self, request):
        """
        use the HTTP client to get a token from the Announce.js server,
        for the currently logged in user.
        """
        res = announce_client.get_token(request.user.pk)
        return res

    def has_announce_cookie(self, request):
        cookie = request.COOKIES.get(announce_cookie_name)
        if cookie:
            return True
        return False

    def determine_path(self, request):
        # Use the same path as the session cookie.
        return settings.SESSION_COOKIE_PATH

    def determine_domain(self, request):
        # Use the same domain as the session cookie.
        return settings.SESSION_COOKIE_DOMAIN

    def set_announce_cookie(self, request, response):
        cookie_path = self.determine_path(request)
        cookie_domain = self.determine_domain(request)
        announce_cookie_value = self.get_token(request)
        
        # calculate expiry
        cookie_age = datetime.timedelta(seconds=settings.SESSION_COOKIE_AGE)

        utc_date = datetime.datetime.utcnow()
        cookie_date_str = '%a, %d-%b-%Y %H:%M:%S GMT'
        expires = datetime.datetime.strftime(
            utc_date + cookie_age, cookie_date_str
        )

        # Set the cookie. use the same path, domain and expiry
        # as the cookie set for the session.
        response.set_cookie(
            announce_cookie_name,
            announce_cookie_value,
            max_age=settings.SESSION_COOKIE_AGE,
            expires=expires,
            path=cookie_path,
            domain=cookie_domain
        )
        return response

    def process_response(self, request, response):
        # We only use it for authenticated users
        if not hasattr(request, 'user'):
            return response
            
        if not request.user.is_authenticated() and \
        announce_cookie_name in request.COOKIES:
            # If there is no authenticated user attached to this request,
            # but the announce.js token cookie is still present, delete it.
            # This is usually called on logout.
            path = self.determine_path(request)
            domain = self.determine_domain(request)
            response.delete_cookie(
                announce_cookie_name, path=path, domain=domain
            )
            return response
        
        # skip unauthenticated users
        if not request.user.is_authenticated():
            return response

        # Check if we have the cookie already set:
        if self.has_announce_cookie(request):
            return response

        # If not, set it.
        self.set_announce_cookie(request, response)
        return response



import httplib
import socket

__version__ = '0.1.4'

class AnnounceClient(object):
    """
    A really basic HTTP client to the Announce.js internal interface.
    It is intended for use with django, but could easily be adapted
    for general use (or maybe for another python web framework)
    """
    def __init__(self):
        from django.conf import settings
        from django.utils import simplejson as json
        self.base_url = getattr(settings, 'ANNOUNCE_API_ADDR', 'localhost:6600')
        self.json = json

    def _do_request(self, method, path, *args, **kwargs):
        # A generic wrapper around httplib.
        # I initally wanted to use @kennethreitz's awesome Requests
        # (python-requests.org) but didn't want to intoduce more dependencies.
        try:
            con = httplib.HTTPConnection(self.base_url)
            con.request(method, path, *args, **kwargs)
            return con.getresponse()
        except (httplib.HTTPException, socket.error):
            return None

    def get_token(self, user_id):
        """
        For a given user ID, return a token. Keep in mind that this ID
        could be anything that uniquley describes a user in the system.
        A primary key from the users table is a good fit, but so is
        a session ID, as long as it is unique.
        """
        path = '/auth/token/%s' % (user_id)
        response = self._do_request('POST', path)
        if not response or response.status > 399:
            return None
        try:
            resp = self.json.loads(response.read())
        except ValueError:
            return None
        if resp:
            if 'token' in resp:
                return '%s|%s' % (user_id, resp['token'])
        return None

    def register_group(self, user_id, group_name):
        """
        Add a user ID (read about what a user ID is under get_token())
        to a given group name. Groups are created on the fly when adding
        the first user. This group could be anything you want it to be,
        no rules here. Map a set of users to a logical name.
        """
        path = '/auth/group/%s/%s' % (group_name, user_id)
        response = self._do_request('POST', path)
        response.read()

    def unregister_group(self, user_id, group_name):
        """
        Remove a user from group.
        """
        path = '/auth/group/%s/%s' % (group_name, user_id)
        response = self._do_request('DELETE', path)
        response.read()


    def emit(self, user_id, channel, data):
        """
        send a message over the given channel to the specified user.
        `data` should be a regular python dict, or any other object
        that is json.dumps'able.
        """
        path = '/emit/user/%s/%s' % (user_id, channel)
        headers = {'Content-Type' : 'application/json'}
        data = self.json.dumps(data)
        response = self._do_request('POST', path, data, headers)
        response.read()

    def broadcast_group(self, group_name, channel, data):
        """
        Emitts the message over the specified channel to all members
        of the group.
        """
        path = '/emit/group/%s/%s' % (group_name, channel)
        headers = {'Content-Type' : 'application/json'}
        data = self.json.dumps(data)
        response = self._do_request('POST', path, data, headers)
        response.read()

    def broadcast_room(self, channel, data):
        """
        Emitts the message over the specified channel to all members
        of the group.
        """
        path = '/emit/room/%s' % (channel)
        headers = {'Content-Type' : 'application/json'}
        data = self.json.dumps(data)
        response = self._do_request('POST', path, data, headers)
        response.read()

    def broadcast(self, channel, data):
        """
        Emit the given message to all connected users.
        """
        path = '/emit/broadcast/%s' % (channel)
        headers = {'Content-Type' : 'application/json'}
        data = self.json.dumps(data)
        response = self._do_request('POST', path, data, headers)
        response.read()

    def get_room_status(self, channel):
        """
        Return information about the current status of a room.
        (most importanly, members currently online in that room).
        """
        path = '/status/room/%s' % (channel)
        response = self._do_request('GET', path)
        data = response.read()
        try:
            return self.json.loads(data)
        except ValueError:
            return None




from setuptools import setup, find_packages

setup(
    name='django-announce',
    version=__import__('announce').__version__,
    author='Oz Katz',
    author_email='oz.katz@ripplify.com',
    description='an Announce.js client for Django',
    url='http://github.com/ozkatz/django-announce/',
    license='BSD',
    packages=find_packages(),
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django'
    ]
)



from django.conf import settings
from django.template import Library
register = Library()

def _determine_base_path():
    client = getattr(settings, 'ANNOUNCE_CLIENT_ADDR', 'localhost:5500')
    secure = getattr(settings, 'ANNOUNCE_HTTPS', False)
    proto = 'http'
    if secure:
        proto = 'https'

    return '%s://%s' % (proto, client)    

@register.simple_tag
def announce_socketio_path():
    return '%s/%s' %(_determine_base_path(), 'socket.io/socket.io.js')

@register.simple_tag
def announce_js_path():
    return '%s/%s' %(_determine_base_path(), 'client/announce.client.js')


@register.simple_tag
def announce_js():
    """
    returns the needed HTML tags for including announce.js and its dependency
    (socket.io) in the client's browser. make sure that `ANNOUNCE_CLIENT_ADDR`
    is in the settings.py file and is reachable from the internet, as
    it will be used as the domain name. if you are using announce.js over
    SSL, be sure to set `ANNOUNCE_HTTPS` to True.
    """
    path = _determine_base_path()
    return """
    <script type="text/javascript" src="%s/socket.io/socket.io.js"></script>
    <script type="text/javascript" src="%s/client/announce.client.js"></script>
    """ % (path, path)